pub struct LongPollId {
pub(crate) tip_height: u32,
pub(crate) tip_hash_checksum: u32,
pub(crate) max_timestamp: u32,
pub(crate) mempool_transaction_count: u32,
pub(crate) mempool_transaction_content_checksum: u32,
}Expand description
The encoded long poll ID, generated from the LongPollInput.
zcashd IDs are currently 69 hex/decimal digits long.
Since Zebra’s IDs are only 46 hex/decimal digits, mining pools should be able to handle them.
Fields
tip_height: u32The tip height used to generate the template containing this long poll ID.
If the tip block height changes, a new template must be provided. Old work is no longer valid.
The height is technically redundant, but it helps with debugging. It also reduces the probability of a missed tip change.
tip_hash_checksum: u32A checksum of the tip hash used to generate the template containing this long poll ID.
If the tip block changes, a new template must be provided. Old work is no longer valid. This checksum is not cryptographically secure.
It’s ok to do a probabilistic check here, so we choose a 1 in 2^32 chance of missing a block change.
max_timestamp: u32The max time in the same template as this long poll ID.
If the max time is reached, a new template must be provided. Old work is no longer valid.
Ideally, a new template should be provided at least one target block interval before the max time. This avoids wasted work.
Zcash times are limited to 32 bits by the consensus rules.
mempool_transaction_count: u32The number of transactions in the mempool when the template containing this long poll ID was generated. This might be different from the number of transactions in the template, due to ZIP-317.
If the number of mempool transactions changes, a new template might be provided. Old work is still valid.
The number of transactions is limited by the mempool DoS limit.
Using the number of transactions makes mempool checksum attacks much harder. It also helps with debugging, and reduces the probability of a missed mempool change.
mempool_transaction_content_checksum: u32A checksum of the effecting hashes of the transactions in the mempool, when the template containing this long poll ID was generated. We ignore changes to authorizing data.
This might be different from the transactions in the template, due to ZIP-317.
If the content of the mempool changes, a new template might be provided. Old work is still valid.
This checksum is not cryptographically secure.
It’s ok to do a probabilistic check here, so we choose a 1 in 2^32 chance of missing a transaction change.
Security
Attackers could use dust transactions to keep the checksum at the same value. But this would likely change the number of transactions in the mempool.
If an attacker could also keep the number of transactions constant, a new template will be generated when the tip hash changes, or the max time is reached.
Implementations
Sourceimpl LongPollId
Auto-generated by derive_getters::Getters.
impl LongPollId
Auto-generated by derive_getters::Getters.
Sourcepub fn tip_height(&self) -> u32
pub fn tip_height(&self) -> u32
The tip height used to generate the template containing this long poll ID.
If the tip block height changes, a new template must be provided. Old work is no longer valid.
The height is technically redundant, but it helps with debugging. It also reduces the probability of a missed tip change.
Sourcepub fn tip_hash_checksum(&self) -> u32
pub fn tip_hash_checksum(&self) -> u32
A checksum of the tip hash used to generate the template containing this long poll ID.
If the tip block changes, a new template must be provided. Old work is no longer valid. This checksum is not cryptographically secure.
It’s ok to do a probabilistic check here, so we choose a 1 in 2^32 chance of missing a block change.
Sourcepub fn max_timestamp(&self) -> u32
pub fn max_timestamp(&self) -> u32
The max time in the same template as this long poll ID.
If the max time is reached, a new template must be provided. Old work is no longer valid.
Ideally, a new template should be provided at least one target block interval before the max time. This avoids wasted work.
Zcash times are limited to 32 bits by the consensus rules.
Sourcepub fn mempool_transaction_count(&self) -> u32
pub fn mempool_transaction_count(&self) -> u32
The number of transactions in the mempool when the template containing this long poll ID was generated. This might be different from the number of transactions in the template, due to ZIP-317.
If the number of mempool transactions changes, a new template might be provided. Old work is still valid.
The number of transactions is limited by the mempool DoS limit.
Using the number of transactions makes mempool checksum attacks much harder. It also helps with debugging, and reduces the probability of a missed mempool change.
Sourcepub fn mempool_transaction_content_checksum(&self) -> u32
pub fn mempool_transaction_content_checksum(&self) -> u32
A checksum of the effecting hashes of the transactions in the mempool, when the template containing this long poll ID was generated. We ignore changes to authorizing data.
This might be different from the transactions in the template, due to ZIP-317.
If the content of the mempool changes, a new template might be provided. Old work is still valid.
This checksum is not cryptographically secure.
It’s ok to do a probabilistic check here, so we choose a 1 in 2^32 chance of missing a transaction change.
Security
Attackers could use dust transactions to keep the checksum at the same value. But this would likely change the number of transactions in the mempool.
If an attacker could also keep the number of transactions constant, a new template will be generated when the tip hash changes, or the max time is reached.
Sourceimpl LongPollId
impl LongPollId
Sourceimpl LongPollId
impl LongPollId
Sourcepub fn submit_old(&self, old_long_poll_id: &LongPollId) -> bool
pub fn submit_old(&self, old_long_poll_id: &LongPollId) -> bool
Returns true if shares using old_long_poll_id can be submitted in response to the
template for self:
https://en.bitcoin.it/wiki/BIP_0022#Optional:_Long_Polling
Old shares may be valid if only the mempool transactions have changed, because newer transactions don’t have to be included in the old shares.
But if the chain tip has changed, the block header has changed, so old shares are invalid. (And if the max time has changed on testnet, the block header has changed.)